@@ -148,6 +148,7 @@ sj_extractor_dispose (GObject *object)
   }
 
   if (priv->pipeline) {
+    gst_element_set_state (priv->pipeline, GST_STATE_NULL);
     g_object_unref (priv->pipeline);
     priv->pipeline = NULL;
   }
@@ -159,14 +160,12 @@ static void
 sj_extractor_finalize (GObject *object)
 {
   SjExtractorPrivate *priv = SJ_EXTRACTOR (object)->priv;
-
-  if (priv->pipeline)
-    gst_element_set_state (priv->pipeline, GST_STATE_NULL);
-
+  
   if (priv->tick_id)
     g_source_remove (priv->tick_id);
 
   g_free (priv->device_path);
+  
   if (priv->construct_error)
     g_error_free (priv->construct_error);
 
@@ -468,6 +467,7 @@ sj_extractor_set_paranoia (SjExtractor *extractor, const int paranoia_mode)
 void
 sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *track, const char* url, GError **error)
 {
+  GParamSpec *spec;
   GstStateChangeReturn state_ret;
   SjExtractorPrivate *priv;
   GstIterator *iter;
@@ -492,7 +492,10 @@ sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *track, c
   }
 
   /* Need to do this, as playback will have locked the read speed to 2x previously */
-  g_object_set (G_OBJECT (priv->cdsrc), "read-speed", G_MAXINT, NULL);
+  spec = g_object_class_find_property (G_OBJECT_GET_CLASS (priv->cdsrc), "read-speed");
+  if (spec && spec->value_type == G_TYPE_INT) {
+    g_object_set (G_OBJECT (priv->cdsrc), "read-speed", ((GParamSpecInt*)spec)->maximum, NULL);
+  }
 
   /* Set the output filename */
   gst_element_set_state (priv->filesink, GST_STATE_NULL);
@@ -560,6 +563,12 @@ sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *track, c
                                  GST_TAG_DATE, track->album->release_date,
                                  NULL);
       }
+      if (track->album->disc_number > 0) {
+        gst_tag_setter_add_tags (tagger,
+                                 GST_TAG_MERGE_APPEND,
+                                 GST_TAG_ALBUM_VOLUME_NUMBER, track->album->disc_number,
+                                 NULL);
+      }
       gst_object_unref (tagger);
       break;
     case GST_ITERATOR_RESYNC:
